"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const config_1 = require("@cli-engine/config");
const cli_flags_1 = require("cli-flags");
const nock = require("nock");
const command_1 = require("./command");
const deps_1 = require("./deps");
class Command extends command_1.Command {
    run() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            deps_1.default.cli.log('foo');
        });
    }
}
Command.id = 'foo:bar';
Command.flags = { myflag: cli_flags_1.flags.boolean() };
Command.args = [{ name: 'myarg', required: false }];
test('it meets the interface', () => {
    let c = Command;
    expect(c).toBeDefined();
});
test('shows the ID', () => {
    expect(Command.id).toEqual('foo:bar');
});
test('runs the command', () => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const { cmd, stdout } = yield Command.mock();
    expect(cmd.flags).toEqual({});
    expect(cmd.argv).toEqual([]);
    expect(stdout).toEqual('foo\n');
}));
test('has stdout', () => tslib_1.__awaiter(this, void 0, void 0, function* () {
    class Command extends command_1.Command {
        run() {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                deps_1.default.cli.stdout.log(this.flags.print);
            });
        }
    }
    Command.flags = { print: cli_flags_1.flags.string(), bool: cli_flags_1.flags.boolean() };
    const { stdout } = yield Command.mock(['--print=foo']);
    expect(stdout).toEqual('foo\n');
}));
test('has stderr', () => tslib_1.__awaiter(this, void 0, void 0, function* () {
    class Command extends command_1.Command {
        run() {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                deps_1.default.cli.stderr.log(this.flags.print);
            });
        }
    }
    Command.flags = { print: cli_flags_1.flags.string() };
    const { stderr } = yield Command.mock(['--print=foo']);
    expect(stderr).toEqual('foo\n');
}));
test('parses args', () => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const { cmd } = yield Command.mock(['one']);
    expect(cmd.flags).toEqual({});
    expect(cmd.argv).toEqual(['one']);
    expect(cmd.args).toEqual({ myarg: 'one' });
}));
test('has help', () => tslib_1.__awaiter(this, void 0, void 0, function* () {
    class Command extends command_1.Command {
        run() {
            return tslib_1.__awaiter(this, void 0, void 0, function* () { });
        }
    }
    Command.id = 'config:get';
    Command.help = `this is

some multiline help
`;
    let config = new config_1.Config();
    expect(Command.buildHelp(config)).toEqual(`Usage: cli-engine config:get

this is

some multiline help\n`);
    expect(Command.buildHelpLine(config)).toEqual(['config:get', undefined]);
}));
describe('http', () => {
    let api;
    let command;
    beforeEach(() => {
        api = nock('https://api.heroku.com');
        nock.disableNetConnect();
        command = new Command(new config_1.Config());
    });
    afterEach(() => {
        api.done();
    });
    test('makes an HTTP request', () => tslib_1.__awaiter(this, void 0, void 0, function* () {
        api = nock('https://api.heroku.com', {
            reqheaders: {
                'user-agent': `cli-engine/0.0.0 (freebsd-x86) node-${process.version}`,
            },
        });
        api.get('/').reply(200, { message: 'ok' });
        let command = new Command(new config_1.Config({ platform: 'freebsd', arch: 'x86' }));
        let { body } = yield command.http.get('https://api.heroku.com');
        expect(body).toEqual({ message: 'ok' });
    }));
    describe('.post', () => tslib_1.__awaiter(this, void 0, void 0, function* () {
        test('makes a post request with body', () => tslib_1.__awaiter(this, void 0, void 0, function* () {
            api.post('/', { karate: 'chop', judo: 'throw', jujitsu: 'strangle' }).reply(200, { message: 'ok' });
            const body = { karate: 'chop', judo: 'throw', jujitsu: 'strangle' };
            yield command.http.post('https://api.heroku.com', { body });
        }));
    }));
    test('stream', () => tslib_1.__awaiter(this, void 0, void 0, function* () {
        api = nock('https://api.heroku.com', {
            reqheaders: {
                'user-agent': `cli-engine/0.0.0 (darwin-x64) node-${process.version}`,
            },
        });
        api.get('/').reply(200, { message: 'ok' });
        let command = new Command(new config_1.Config({ platform: 'darwin', arch: 'x64' }));
        const { response } = yield command.http.stream('https://api.heroku.com');
        const body = JSON.parse(yield concat(response));
        expect(body).toEqual({ message: 'ok' });
    }));
    function concat(stream) {
        return new Promise(resolve => {
            let strings = [];
            stream.on('data', data => strings.push(data));
            stream.on('end', () => resolve(strings.join('')));
        });
    }
});
test('allows deprecated input', () => tslib_1.__awaiter(this, void 0, void 0, function* () {
    class Command extends command_1.Command {
        run() {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                deps_1.default.cli.log(this.flags.myflag);
            });
        }
    }
    Command.flags = { myflag: cli_flags_1.flags.string() };
    const { stdout } = yield Command.mock('--myflag', 'foo');
    expect(stdout).toEqual('foo\n');
}));
